#include "file.h"


file::file ()
  : fp ( NULL ), 
#ifdef HAVE_LIBZ
  gzfp ( NULL ),
#endif
  f_open ( false ),
  gz_open ( false ),
  _mode ( 0 )
{
}

file::~file ()
{
  close ();
}

void file::open ( char* path, int mode )
{
  _mode = mode;
#ifdef HAVE_LIBZ
  if ( !filtered () && !huffmanOnly ()
       && ( read () && write () )
       || ( write () && ( compression () == file::NO_COMPRESSION )  )
       )
    {
#endif
      // normal file
      strstream mode_strm;
      if ( _mode | file::READ == _mode )
	mode_strm << "r";
     
      if ( _mode | file::WRITE == _mode )
	mode_strm << "w";

      if ( _mode | file::BINARY == _mode )
	mode_strm << "b";

      mode_strm << ends;

      fp = fopen ( path, mode_strm.str () );
      if ( fp == NULL )
	cerr << __FILE__ << ": " << __LINE__ 
	     << ": Error opening file: " << path << endl;
      else
	{
	  f_open = true;
	}
#ifdef HAVE_LIBZ
    }
  else if ( ( read () != write () && ( read () || write () ) )
	    && ( ( filtered () && !huffmanOnly () )
		 || ( huffmanOnly () && !filtered () )
		 || ( !huffmanOnly () && !filtered () ) )
	    )
    {
      // compressed file
      strstream mode_strm;
      if ( _mode | file::READ == _mode )
	mode_strm << "rb";
      else
	mode_strm << "wb";

      mode_strm << compression ();
      
      if ( filtered () )
	mode_strm << "f";
      else if ( huffmanOnly () )
	mode_strm << "h";
       
      mode_strm << ends;

      gzfp = gzopen ( path, mode_strm.str () );
      if ( gzfp == NULL )
	cerr << __FILE__ << ": " << __LINE__ 
	     << ": Error opening file: " << path << endl;
      else
	{
	  gz_open = true;
	}
    }
  else
    cerr << __FILE__ << ": " << __LINE__ 
	 << ": Invalid mode\n";
#endif

}


void file::open ( int fd, int mode )
{
  _mode = mode;
#ifdef HAVE_LIBZ
  if ( !filtered () && !huffmanOnly ()
       && ( read () && write () )
       || ( write () && compression () == file::NO_COMPRESSION )
       )
    {
#endif
      // normal file
      strstream mode_strm;
      if ( _mode | file::READ == _mode )
	mode_strm << "r";
     
      if ( _mode | file::WRITE == _mode )
	mode_strm << "w";

      if ( _mode | file::BINARY == _mode )
	mode_strm << "b";

      mode_strm << ends;

      fp = fdopen ( fd, mode_strm.str () );
      if ( fp == NULL )
	cerr << __FILE__ << ": " << __LINE__ 
	     << ": Error opening file\n";
      else
	f_open = true;
#ifdef HAVE_LIBZ
    }
  else if ( ( read () != write () && ( read () || write () ) )
	    && ( ( filtered () && !huffmanOnly () )
		 || !huffmanOnly () && filtered () )
	    )
    {
      // compressed file
      strstream mode_strm;
      if ( _mode | file::READ == _mode )
	mode_strm << "rb";
      else
	mode_strm << "wb";

      mode_strm << compression ();
      
      if ( filtered () )
	mode_strm << "f";
      else if ( huffmanOnly () )
	mode_strm << "h";
       
      mode_strm << ends;

      gzfp = gzdopen ( fd, mode_strm.str () );
      if ( gzfp == NULL )
	cerr << __FILE__ << ": " << __LINE__ 
	     << ": Error opening file\n";
      else
	gz_open = true;
    }
  else
    cerr << __FILE__ << ": " << __LINE__ 
	 << ": Invalid mode\n";
#endif
}

  
void file::setCompression ( int level, int strategy )
{
#ifdef HAVE_LIBZ
  gzsetparams ( gzfp, level, strategy / file::COMPRESSION_MOD );
#endif
}

int file::read ( void* buf, unsigned len )
{
#ifdef HAVE_LIBZ
  if ( gz_open )
    {
      return gzread ( gzfp, buf, len );
    }
  else
#endif 
    if ( f_open )
      {
	return fread ( buf, len, 1, fp );
      }
    else
      return -1;
}

int file::write ( void* buf, unsigned len )
{
#ifdef HAVE_LIBZ
  if ( gz_open )
    return ( gzwrite ( gzfp, buf, len ) );
  else
#endif
    if ( f_open )
      return ( fwrite ( buf, len, 1, fp ) );
    else
      return -1;
}

int file::flush ( int type )
{
#ifdef HAVE_LIBZ
  if ( gz_open )
    return ( gzflush ( gzfp, type ) );
  else 
#endif
    if ( f_open )
      return ( fflush ( fp ) );
    else
      return -1;
}

long file::seek ( long offset, int whence )
{
#ifdef HAVE_LIBZ
  if ( gz_open )
    return ( gzseek ( gzfp, offset, whence ) );
  else
#endif
    if ( f_open )
      return ( fseek ( fp, offset, whence ) );
    else
      return -1;
}

long file::tell ()
{
#ifdef HAVE_LIBZ
  if ( gz_open )
    return ( gztell ( gzfp ) );
  else
#endif
    if ( f_open )
      return ( ftell ( fp ) );
    else
      return -1;
}

long file::rewind ()
{
#ifdef HAVE_LIBZ
  if ( gz_open )
    return ( gzrewind ( gzfp ) );
  else 
#endif
    if ( f_open )
      {
	::rewind ( fp );
	return ( tell () );
      }
    else
      return -1;  
}

bool file::eof ()
{
#ifdef HAVE_LIBZ
  if ( gz_open )
    return ( gzeof ( gzfp ) );
  else 
#endif
    if ( f_open )
      return ( feof ( fp ) );
    else
      return -1;  
}

int file::close ()
{
#ifdef HAVE_LIBZ
  if ( gz_open )
    {
      gz_open = false;
      return ( gzclose ( gzfp ) );
    }
  else 
#endif
    if ( f_open )
      {
	f_open = false;
	return ( feof ( fp ) );
      }
    else
      return -1;  
}

int file::compression ()
{
  return _mode % file::COMPRESSION_MOD;
}

bool file::filtered ()
{
  return ( _mode | file::FILTERED ) == _mode;
}

bool file::huffmanOnly ()
{
  return ( _mode | file::HUFFMAN_ONLY ) == _mode;
}

bool file::binary ()
{
  return ( _mode | file::BINARY ) == _mode;
}

bool file::read ()
{
  return ( _mode | file::READ ) == _mode;
}

bool file::write ()
{
  return ( _mode | file::WRITE ) == _mode;
}

bool file::open ()
{
  return f_open
#ifdef HAVE_LIBZ
    || gz_open
#endif
    ;
}

void file::clearErr ()
{
  if ( f_open )
    clearerr ( fp );
}
